package tech.riemann.ims.controller.platform.material;

import club.zhcs.lina.starter.exception.BizException;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.nutz.lang.Strings;
import org.nutz.lang.random.R;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import tech.riemann.ims.entity.material.Material;
import tech.riemann.ims.service.material.IMaterialService;

import java.util.List;

/**
 * @author mayong
 * @since 2024/11/27 15:11
 */
@RestController
@Tag(name = "Material", description = "物料管理")
@RequiredArgsConstructor
public class MaterialController {

    private final IMaterialService materialService;

    @GetMapping("materials")
    @Operation(summary = "分页查询物料列表")
    public IPage<Material> materials(
            @Parameter(description = "页码") @RequestParam(required = false, defaultValue = "1") int page,
            @Parameter(description = "页面大小") @RequestParam(required = false, defaultValue = "10") int size,
            @Parameter(description = "类型") @RequestParam(required = false, defaultValue = "") Integer type,
            @Parameter(description = "搜索关键词") @RequestParam(required = false, defaultValue = "") String key) {
        boolean hasLike = Strings.isNotBlank(key);
        key = String.format("%%%s%%", key);
        return materialService.page(Page.of(page, size),Wrappers.<Material>lambdaQuery()

                .like(hasLike, Material::getName, key)
                .or()
                .like(hasLike, Material::getCode, key)
                .eq(type != null, Material::getType, type)
                .orderByDesc(Material::getUpdatedTime));
    }

    @GetMapping("material/list")
    @Operation(summary = "查询所有物料列表")
    public List<Material> all(){
        return materialService.list();
    }

    @GetMapping("material/{id}")
    @Operation(summary = "物料详情")
    public Material detail(@Parameter(description = "物料id", required = true) @PathVariable long id) {
        return materialService.getById(id);
    }

    @PutMapping("material")
    @Operation(summary = "增加/编辑物料")
    public Material saveOrUpdateMaterial(@Validated @Parameter(description = "物料信息") @RequestBody Material material) {
        if (material.getId() == null || material.getId() <= 0) {
            material.setCode(R.UU16());
            if (materialService.save(material)) {
                return material;
            } else {
                throw BizException.create("保存物料失败!");
            }
        } else {
            if (materialService.update(material, Wrappers.<Material> lambdaUpdate().eq(Material::getId, material.getId()))) {
                return material;
            } else {
                throw BizException.create("更新物料失败!");
            }
        }
    }

    @DeleteMapping("material/{id}")
    @Operation(summary = "删除物料")
    @ResponseStatus(HttpStatus.OK)
    public void deleteMaterial(@Parameter(description = "物料id", required = true) @PathVariable long id) {
        if (!materialService.removeById(id)) {
            throw BizException.create("删除物料失败!");
        }
    }
}
