package tech.riemann.ims.entity.material;

import club.zhcs.lina.utils.enums.Codebook;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import lombok.experimental.FieldNameConstants;
import lombok.experimental.SuperBuilder;
import org.nutz.dao.entity.annotation.*;
import org.nutz.json.JsonField;
import tech.riemann.ims.entity.IdBaseEntity;
import tech.riemann.ims.enums.ApplyTypeEnum;
import tech.riemann.ims.enums.AuditTypeEnum;
import tech.riemann.ims.enums.HandleEnum;

import java.io.Serial;
import java.time.LocalDateTime;
import java.util.Optional;

/**
 * @author mayong
 * @since 2024/11/26 16:53
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@TableName("t_apply_form")
@Table("t_apply_form")
@Comment("入库/出库/盘点 申请单")
@Schema(name = "ApplyForm", description = "入库/出库/盘点 申请单")
public class ApplyForm extends IdBaseEntity {


    @Serial
    private static final long serialVersionUID = 1L;

    @Schema(description = "类型（1: 采购入库申请 2: 归还入库申请 3: 出库外借申请 4: 盘点申请）", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("af_type")
    @Column("af_type")
    @Comment("类型")
    @ColDefine(type = ColType.INT)
    private ApplyTypeEnum type;

    @Schema(description = "盘点类型（1: 全盘 2: 部分盘点）", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("af_audit_type")
    @Column("af_audit_type")
    @Comment("盘点类型")
    @ColDefine(type = ColType.INT)
    private AuditTypeEnum auditType;

    @Schema(description = "申请人", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("af_applicant")
    @Column("af_applicant")
    @Comment("申请人")
    @ColDefine(notNull = false, width = 128, precision = 0)
    private String applicant;

    @Schema(description = "申请日期", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("af_apply_date")
    @Column("af_apply_date")
    @Comment("申请日期")
    @ColDefine(type = ColType.DATE, notNull = false, width = 128, precision = 0)
    private LocalDateTime applyDate;

    @Schema(description = "是否确认（0: 未确认 1: 已确认）", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("af_is_confirm")
    @Column("af_is_confirm")
    @Comment("是否确认")
    @ColDefine(type = ColType.BOOLEAN, notNull = false, width = 128, precision = 0)
    private Boolean isConfirm;

    @Schema(description = "结果(系统自动生成)", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("af_result")
    @Column("af_result")
    @Comment("结果(系统自动生成)")
    @ColDefine(notNull = false, width = 512, precision = 0)
    private String result;

    @Schema(description = "异常原因", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("af_exception")
    @Column("af_exception")
    @Comment("异常原因")
    @ColDefine(type = ColType.VARCHAR, notNull = false, width = 512, precision = 0)
    private String exception;

    @Schema(description = "处理方式(0-无需处理 1-忽略并修改库存 2-补充库存 3-其他)", requiredMode = Schema.RequiredMode.NOT_REQUIRED)
    @TableField("s_exception_handle")
    @Column("s_exception_handle")
    @Comment("处理方式(0-无需处理 1-忽略并修改库存 2-补充库存 3-其他)")
    @ColDefine(type = ColType.VARCHAR, notNull = false, width = 128, precision = 0)
    private HandleEnum handle;

    @JsonField
    public Codebook getTypeInfo() {
        return Optional.of(getType()).orElse(ApplyTypeEnum.PURCHASE_RECEIPT).build();
    }

    @JsonField
    public Codebook getAuditTypeInfo() {
        return Optional.of(getAuditType()).orElse(AuditTypeEnum.ALL).build();
    }

    @JsonField
    public Codebook getHandleInfo() {
        return Optional.of(getHandle()).orElse(HandleEnum.NO_ACTION).build();
    }
}
